@@ -3,6 +3,7 @@ from __future__ import annotations
 import logging
 from time import time
 from typing import Any
+from asyncio import CancelledError
 
 import pytest
 
@@ -176,6 +177,7 @@ async def test_queue(pool_cls, dsn):
     async def worker(n):
         t0 = time()
         async with p.connection() as conn:
+            assert conn._pool is p
             await conn.execute("select pg_sleep(0.2)")
             pid = conn.info.backend_pid
         t1 = time()
@@ -684,7 +686,7 @@ async def test_cancellation_in_queue(pool_cls, dsn):
         tasks = [spawn(worker, (i,)) for i in range(nconns * 3)]
 
         # wait until the pool has served all the connections and clients are queued.
-        await ev.wait_timeout(3.0)
+        assert await ev.wait_timeout(3.0)
         for i in range(10):
             if p.get_stats().get("requests_queued", 0):
                 break
@@ -705,6 +707,92 @@ async def test_cancellation_in_queue(pool_cls, dsn):
             assert await cur.fetchone() == (1,)
 
 
+@skip_sync
+async def test_cancel_on_check(pool_cls, dsn):
+    do_cancel = True
+
+    async def check(conn):
+        nonlocal do_cancel
+        if do_cancel:
+            do_cancel = False
+            raise CancelledError()
+
+        await pool_cls.check_connection(conn)
+
+    async with pool_cls(
+        dsn, min_size=min_size(pool_cls, 1), check=check, timeout=1.0
+    ) as p:
+        try:
+            async with p.connection() as conn:
+                await conn.execute("select 1")
+        except CancelledError:
+            pass
+
+        async with p.connection() as conn:
+            await conn.execute("select 1")
+
+
+@skip_sync
+async def test_cancel_on_rollback(pool_cls, dsn, monkeypatch):
+    do_cancel = False
+
+    async with pool_cls(dsn, min_size=min_size(pool_cls, 1), timeout=1.0) as p:
+        async with p.connection() as conn:
+
+            async def rollback(self):
+                if do_cancel:
+                    raise CancelledError()
+                else:
+                    await type(self).rollback(self)
+
+            monkeypatch.setattr(type(conn), "rollback", rollback)
+            await conn.execute("select 1")
+
+        do_cancel = True
+        with pytest.raises((psycopg.errors.SyntaxError, CancelledError)):
+            async with p.connection() as conn:
+                await conn.execute("selexx 2")
+
+        do_cancel = False
+        async with p.connection() as conn:
+            cur = await conn.execute("select 3")
+            assert (await cur.fetchone()) == (3,)
+
+
+@pytest.mark.crdb_skip("backend pid")
+async def test_drain(pool_cls, dsn):
+    pids1 = set()
+    pids2 = set()
+    pids3 = set()
+    async with pool_cls(dsn, min_size=min_size(pool_cls, 2)) as p:
+        await p.wait()
+
+        async with p.connection() as conn:
+            pids1.add(conn.info.backend_pid)
+            async with p.connection() as conn2:
+                pids1.add(conn2.info.backend_pid)
+                await p.drain()
+        assert len(pids1) == 2
+
+        async with p.connection() as conn:
+            pids2.add(conn.info.backend_pid)
+            async with p.connection() as conn2:
+                pids2.add(conn2.info.backend_pid)
+
+        assert len(pids2) == 2
+
+        assert not pids1 & pids2
+
+        async with p.connection() as conn:
+            pids3.add(conn.info.backend_pid)
+            async with p.connection() as conn2:
+                pids3.add(conn2.info.backend_pid)
+
+        assert len(pids3) == 2
+        if pool_cls is not pool.AsyncNullConnectionPool:
+            assert pids2 == pids3
+
+
 def min_size(pool_cls, num=1):
     """Return the minimum min_size supported by the pool class."""
     if pool_cls is pool.AsyncConnectionPool:
